#
# spec file for package docker-runc
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#
# nodebuginfo


# MANUAL: Update the git_version, git_short, and git_revision
%define git_version 3f2f8b84a77f73d38244dd690525642a72156c64
%define git_short   3f2f8b84a77f
# How to get the git_revision
# git clone ${url}.git runc-upstream
# cd runc-upstream
# git checkout $git_version
# git_revision=r$(git rev-list HEAD | wc -l)
%define git_revision r3338

%define go_tool go
%define _name runc
%define project github.com/opencontainers/%{_name}

# enable libseccomp for sle >= sle12sp2
%if 0%{?sle_version} >= 120200
%define with_libseccomp 1
%endif
# enable libseccomp for leap >= 42.2
%if 0%{?leap_version} >= 420200
%define with_libseccomp 1
%endif
# enable libseccomp for Factory
%if 0%{?suse_version} > 1320
%define with_libseccomp 1
%endif

Name:           docker-runc
Version:        1.0.0rc4+git%{git_revision}_%{git_short}
Release:        lp150.4.2
Summary:        Tool for spawning and running OCI containers
License:        Apache-2.0
Group:          System/Management
Url:            https://github.com/opencontainers/runc
Source:         %{name}-git.%{git_version}.tar.xz
# SUSE-FIX-UPSTREAM: Backport of https://github.com/opencontainers/runc/pull/1555. bsc#1053532
Patch100:       bsc1053532-0001-makefile-drop-usage-of-install.patch
BuildRequires:  fdupes
BuildRequires:  go-go-md2man
BuildRequires:  libapparmor-devel
BuildRequires:  golang(API) = 1.7
%if 0%{?with_libseccomp}
BuildRequires:  libseccomp-devel
%endif
BuildRequires:  libselinux-devel
Recommends:     criu
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
Obsoletes:      runc <= 1.0
# We provide a git revision so that Docker can require it properly.
Provides:       %{name}-git = %{git_version}
ExcludeArch:    s390

%description
runc is a CLI tool for spawning and running containers according to the OCI
specification. It is designed to be as minimal as possible, and is the workhorse
of Docker. It was originally designed to be a replacement for LXC within Docker,
and has grown to become a separate project entirely.

%package test
Summary:        Test package for runc
Group:          System/Management
BuildRequires:  golang(API) = 1.7
Requires:       go-go-md2man
Requires:       libapparmor-devel
%if 0%{?with_libseccomp}
BuildRequires:  libseccomp-devel
%endif
Requires:       libselinux-devel
Recommends:     criu
BuildArch:      noarch
Obsoletes:      runc-test <= 1.0

%description test
Test package for runc. It contains the source code and the tests.

%prep
%setup -q -n %{name}-git.%{git_version}
# bsc#1053532
%patch100 -p1

%build
# Do not use symlinks. If you want to run the unit tests for this package at
# some point during the build and you need to directly use go list directly it
# will get confused by symlinks.
export GOPATH=${HOME}/go
export PROJECT=${HOME}/go/src/%project
mkdir -pv $PROJECT
rm -rf $PROJECT/*
cp -av * $PROJECT

# Additionally enable seccomp.
%if 0%{?with_libseccomp}
export EXTRA_BUILDTAGS+="seccomp"
export EXTRA_GCCFLAGS+="-lseccomp"
%endif

# Build all features.
export BUILDTAGS="apparmor selinux $EXTRA_BUILDTAGS"
export BUILDFLAGS="-buildmode=pie -gccgoflags='-Wl,--add-needed -Wl,--no-as-needed -static-libgo -ldl -lselinux -lapparmor $EXTRA_GCCFLAGS'"

(cat <<EOF
export GOPATH="$GOPATH"
export BUILDTAGS="$BUILDTAGS"
export BUILDFLAGS="$BUILDFLAGS"
EOF
) >./.runc_build_env
source ./.runc_build_env

# Build runc.
make -C $PROJECT EXTRA_FLAGS="-x $BUILDFLAGS" BUILDTAGS="$BUILDTAGS" COMMIT_NO=%{git_version} runc
cp $PROJECT/runc %{name}-%{version}

# Build man pages, this can only be done on arches where we can build go-md2man.
man/md2man-all.sh

%check
source ./.runc_build_env
cd $HOME/go/src/%project

PKG_LIST=$(go list ./... \
		| grep -v '%{project}/libcontainer/cgroups/fs$' \
		| grep -v '%{project}/libcontainer$' \
		| grep -v '%{project}/libcontainer/integration$' \
		| grep -v '%{project}/libcontainer/label$' \
		| grep -v '%{project}/libcontainer/nsenter$' \
		| grep -v '%{project}/libcontainer/user$' \
		| grep -v '%{project}/libcontainer/xattr$' \
%if ! 0%{?with_libseccomp}
		| grep -v '%{project}/libcontainer/seccomp$' \
		| grep -v 'github.com/seccomp/libseccomp-golang$' \
%endif
		)
%go_tool test -buildmode=pie -tags "$BUILDTAGS" -timeout 3m -v $PKG_LIST

%install
source ./.runc_build_env

# Make sure we install in /usr/sbin/docker-runc
install -D -m755 %{name}-%{version} %{buildroot}%{_sbindir}/%{name}
install -d -m755 %{buildroot}/usr/src/%{name}/
cp -av $HOME/go/src/%{project}/* %{buildroot}/usr/src/%{name}/
rm -rf %{buildroot}/usr/src/docker-runc/runc

# We have to rename the man pages to docker-runc.
install -d -m755 %{buildroot}%{_mandir}/man8
cd man/man8
for mp in $(ls runc*.8); do
	install -m644 ${mp} %{buildroot}%{_mandir}/man8/docker-${mp}
done

%fdupes %{buildroot}

%files
%defattr(-,root,root)
%doc README.md LICENSE
%{_sbindir}/docker-runc
%{_mandir}/man8/docker-runc*.8.gz

%files test
%defattr(-,root,root)
/usr/src/docker-runc/

%changelog
* Thu Feb  1 2018 ro@suse.de
- do not build on s390, only on s390x (no go on s390)
* Wed Dec 27 2017 dimstar@opensuse.org
- Fix build with RPM 4.14: exclude is not meant for files to NOT be
  packaged, but should only be used if the files are to be excluded
  from a glob when they end up in a different package. Rather
  remove the unwanted files in the install section.
* Mon Dec 11 2017 asarai@suse.com
- Update to docker-runc 0351df1c5a66838d0c392b4ac4cf9450de844e2d requirement
  for Docker v17.09.1-ce. This also includes a switch to the upstream
  opencontainers sources.
* Mon Oct 16 2017 asarai@suse.com
- Drop backport of https://github.com/opencontainers/runc/pull/1603. bsc#1055676
  It's going to be fixed in Docker instead.
  - bsc1055676-0001-rootfs-preserve-old-mount-flags-when-remounting-bind.patch
* Mon Oct  9 2017 asarai@suse.com
- Update to docker-runc 2d41c047c83e09a6d61d464906feb2a2f3c52aa4, which is
  required for Docker v17.07-ce.
- Add backport of https://github.com/opencontainers/runc/pull/1603. bsc#1055676
  + bsc1055676-0001-rootfs-preserve-old-mount-flags-when-remounting-bind.patch
- Remove fix for CVE-2016-9962, as the patches are now included in the upstream
  source. bsc#1012568
  - CVE-2016-9962.patch
* Tue Sep  5 2017 thipp@suse.de
- Update bsc1053532-0001-makefile-drop-usage-of-install.patch
  + replace missing target "all" with "runc"
* Sun Aug 13 2017 asarai@suse.com
- Use the upstream Makefile, to ensure that we always include the version and
  commit information in runc. This was confusing users (and Docker).
  bsc#1053532
- Add a backported patch to fix a Makefile bug. This also includes some other
  changes to make the docker-runc.spec mirror the newer runc.spec (which
  required additional patching to the Makefile).
  https://github.com/opencontainers/runc/pull/1555
  + bsc1053532-0001-makefile-drop-usage-of-install.patch
* Wed Aug  2 2017 asarai@suse.com
- Use -buildmode=pie for tests and binary build. bsc#1048046 bsc#1051429
- Cleanup seccomp builds similar to bsc#1028638
- Remove the usage of 'cp -r' to reduce noise in the build logs.
* Thu Jul 13 2017 thipp@suse.de
- Remove empty %%post
- Use Obsoletes instead of Conflicts
* Thu Jul  6 2017 thipp@suse.de
- rename runc to docker-runc (bsc#1069758)
* Thu May  4 2017 jmassaguerpla@suse.com
- fix the golang requirement to 1.7 to the subpackages
* Tue May  2 2017 jmassaguerpla@suse.com
- fix golang requirement to 1.7
* Fri Apr 28 2017 jengelh@inai.de
- Substitute %%__-type macro indirections
* Thu Apr 13 2017 jmassaguerpla@suse.com
- update version to the one required by docker-17.04.0-ce (bsc#1034053)
  remove ignore_cgroup2_mountpoint.patch . This is already included in
  the upstream source code.
* Wed Apr 12 2017 jmassaguerpla@suse.com
- Make sure this is being built with go 1.7
* Tue Apr 11 2017 jmassaguerpla@suse.com
- remove the go_arches macro because we are using go1.7 which
  is available in all archs
* Wed Mar 29 2017 jmassaguerpla@suse.com
- fix bsc#1028113 - runc: make sure to ignore cgroup v2 mountpoints
  This is a backport of https://github.com/opencontainers/runc/pull/1266
  + ignore_cgroup2_mountpoint.patch
* Fri Feb 24 2017 jmassaguerpla@suse.com
- update to docker-1.13.0 requirement
* Fri Jan 13 2017 jmassaguerpla@suse.com
- fix CVE-2016-9962 bsc#1012568 and applying the patch
  CVE-2016-9962.patch, because 1.12.6 partially fixes it (it contains
  the first patch attached in bsc#1012568)
* Mon Dec 19 2016 jmassaguerpla@suse.com
- update runc to the version used in docker 1.12.5 (bsc#1016307).
  This fixes bsc#1015661
* Mon Dec 19 2016 asarai@suse.com
- For the moment, we have to switch to using Docker's fork of runC. This *will*
  be solved properly by creating a new package purely for Docker's runC fork,
  because it's quite silly to tie OCI project releases to Docker's vendoring
  scheme. Once this is fixed, this package will be switch to being purely-OCI.
* Fri Dec 16 2016 jmassaguerpla@suse.com
- add the /usr/bin/docker-run symlink to partially fix bsc#1015661
* Thu Nov 24 2016 jmassaguerpla@suse.com
- fix version by adding a revision "counter" so that it will always
  increase
  fix bsc#1009961
* Thu Oct 13 2016 jmassaguerpla@suse.com
- update to 02f8fa7 because that is the needed version for docker 1.12.1 (bsc#1004490)
* Wed Sep 21 2016 jengelh@inai.de
- Run fdupes.
* Mon Sep 19 2016 jmassaguerpla@suse.com
- fix go_arches definition: use global instead of define, otherwise
  it fails to build
* Fri Aug 26 2016 asarai@suse.com
- Remove docker-runc symlink because it's been fixed within the Docker
  package. bsc#978260
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Create a symlink /usr/sbin/docker-runc -> /usr/sbin/docker
  Docker expects this symlink to exist bsc#978260
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Remove GOPATH at the end of the GOPATH assignment
  cause GOPATH is empty and if we do that, we get the path ""
  appended, which causes gcc6-go to complain
* Wed Aug 24 2016 jmassaguerpla@suse.com
- add go_arches in project configuration: this way, we can use the
  same spec file but decide in the project configuration if to
  use gc-go or gcc-go for some archs.
* Thu Aug 18 2016 jmassaguerpla@suse.com
- use gcc6-go instead of gcc5-go (bsc#988408)
- build ppc64le with gc-go because this version builds with gc-go 1.6
* Thu Aug 18 2016 cbrauner@suse.de
- bump git commit id to the one required by docker v1.12.0 (bsc#995058)
- run unit tests during package build
- remove seccomp-use-pkg-config.patch
  The patch is now upstream.
- remove GO_BUILD_FLAGS macro and substitute with BUILDFLAGS env variable to
  allow for easier string appending.
- only run unit test on architectures that provide the go list and go test tools
* Wed Aug 17 2016 cbrauner@suse.de
- Add runc-test package which contains the source code and the test. This
  package will be used to run the integration tests.
- Simplify package build and check sections: Instead of symlinking we default to
  cp -avr. go list gets confused by symlinks hence, we need to copy the source
  code anyway if we want to run unit tests during package build at some point.
* Fri Apr 29 2016 asarai@suse.de
  * Update to runC 0.1.1. (bsc#989566 FATE#320763) Changelog from upstream:
  This release includes a bug fix for adding the selinux mount label in the specification.
* Tue Apr 19 2016 asarai@suse.de
  * Don't use gcc-go for aarch64, since gc has grown support for it and is more
  stable.
* Fri Apr 15 2016 asarai@suse.de
  * Disable seccomp entirely for aarch64 builds, since it is not provided on all
  SUSE platforms.
* Wed Apr 13 2016 asarai@suse.de
  * Update to runC 0.1.0. Changelog from upstream:
  This release updates runc to the OCI runtime specification v0.5.0 and includes
  various fixes and features.
  Features:
  + cgroups: pid limits and stats
  + cgroups: kmem stats
  + systemd cgroup support
  + libcontainer specconv package
  + no pivot root option
  + numeric ids are treated as uid/gid
  + hook improvements
  Bug Fixes:
  * log flushing
  * atomic pid file creation
  * init error recovery
  * seccomp logging removed
  * delete container on aborted start
  * /dev bind mount handling
* Wed Mar 30 2016 asarai@suse.de
  * Install to /usr/sbin.  https://github.com/opencontainers/runc/pull/702
* Sun Mar 27 2016 asarai@suse.de
  * Added runC man pages.
  * Recommended criu, since it's required for the checkpoint and restore
  functionality.
* Sun Mar 27 2016 asarai@suse.de
  * Small updates to method of compilation to better match Makefile.
* Mon Mar 21 2016 asarai@suse.de
  * Make compilation work on gcc-go only systems (ppc and s390).
* Mon Mar 21 2016 asarai@suse.de
  * initial import of runC 0.0.9
  * add patch seccomp-use-pkg-config.patch which allows us to build runC, since
  they assume that the seccomp.h file lives at /usr/include/seccomp.h.
